#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Author: Chris Oliver (excid3@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA


from distutils.core import setup
from lib import consts
import os, os.path, sys, shutil

try:    import py2exe # In case you want to just do a "./setup.py clean"
except: pass

binDir = 'bin'
distDir = 'dist'
buildDir = 'build'
ver = '%s-%s' % (consts.appNameShort, consts.appVersion)
winDir = 'win32'
linDir = 'linux'

manifest = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<assembly xmlns="urn:schemas-microsoft-com:asm.v1" manifestVersion="1.0">
    <assemblyIdentity version="0.64.1.0" processorArchitecture="x86" 
    name="Controls" type="win32"/>
    <description>%s</description>
    <dependency>
        <dependentAssembly>
            <assemblyIdentity type="win32" 
            name="Microsoft.Windows.Common-Controls" version="6.0.0.0" 
            processorArchitecture="X86" publicKeyToken="6595b64144ccf1df"
            language="*"/>
        </dependentAssembly>
    </dependency>
</assembly>
"""%consts.appName

setupOptions = { 
    'name': consts.appName,
    'author': consts.authors,
    'author_email': consts.email,
    'description': consts.description,
    'version': consts.appVersion,
    'url': consts.urlHomepage,
    'license': consts.license,
    'packages': ['lib', 'lib.wxkeryx'] + 
        [subpackage for subpackage in ('lib', 'lib.wxkeryx')],
    'scripts': ['keryx.py'],
    }

# Configure for py2exe
if sys.argv[1] == 'py2exe':
    setupOptions.update({
        'windows' : [{ 'script' : 'keryx.py', 
            'other_resources' : [(24, 1, manifest)],
            'icon_resources': [(1, 'pixmaps/keryx.ico')]}],
        'options' : {'py2exe' : {
            'compressed' : 1, 
            'optimize' : 2, 
            # We need to explicitly include these packages because they 
            # are imported implicitly:
            'packages' : ['commands', 'gzip'], }}})


if __name__ == '__main__':
    #Clean up all .py* files that exist
    print "Cleaning directories"
    for root, dirs, files in os.walk(os.getcwd()):
        for name in files:
            if name.endswith('.pyc') or name.endswith('.pyo'):
                os.remove(os.path.join(root, name))

    if sys.argv[1] == 'clean': sys.exit()

    # Begin setup
    setup(**setupOptions)

    # Remove mess py2exe makes
    if sys.argv[1] == 'py2exe':
        shutil.rmtree(os.path.join(os.getcwd(), buildDir))
        #shutil.rmtree(consts.dirLog)
        #shutil.rmtree(consts.dirProjects)
        if os.path.exists(binDir): shutil.rmtree(binDir)

        # Copy pixmaps and plugins folder to dist
        shutil.copytree(consts.dirPlugins, os.path.join(distDir, 'plugins'))
        shutil.copytree(consts.dirPixmaps, os.path.join(distDir, 'pixmaps'))

        # Rename and move dist directory
        os.rename(distDir, winDir)
        shutil.move(winDir, os.path.join(os.path.join(binDir, ver),winDir))
       
        # Copy doc dir to bin
        shutil.copytree('doc', os.path.join(os.path.join(binDir, ver), 'doc'))
        
        # Create linux directory
        #linDir = os.path.join(os.path.join(binDir, ver), linDir)
        #os.mkdir(linDir)
        #shutil.copyfile('keryx.py', linDir)
        #shutil.copytree('lib', linDir)
        #shutil.copytree('pixmaps', linDir)
        #shutil.copytree('plugins', linDir)

        print "\n\n\nREMEMBER!!! Copy msvcp71.dll into win32 directory before releasing."
